import 'dart:async';
import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';
import 'package:web_socket_channel/web_socket_channel.dart';
import 'package:flutter/material.dart';

import 'package:wowpay_flutter/utils/constants.dart';
import 'dart:io' show Platform;
import '../../wowpay_flutter.dart';

class ModalPayment extends StatefulWidget{
  final String urlInnerCard;
  final String orderId;
  ModalPayment({required this.urlInnerCard,required this.orderId});

  @override
  State<StatefulWidget> createState() {
    // TODO: implement createState
    return _ModalPayment();
  }
}

class _ModalPayment extends State<ModalPayment>{
  final GlobalKey webViewKey = GlobalKey();
  late InAppWebViewController webViewController;
  InAppWebViewGroupOptions options = InAppWebViewGroupOptions(
      crossPlatform: InAppWebViewOptions(
          useShouldOverrideUrlLoading: true,
          supportZoom:false,
          javaScriptCanOpenWindowsAutomatically:true,
          mediaPlaybackRequiresUserGesture: false,
      ),
      android: AndroidInAppWebViewOptions(
        useHybridComposition: true,
      ),
      ios: IOSInAppWebViewOptions(
        allowsInlineMediaPlayback: true,

      ));


  String url = "";
  double progress = 0;
  bool isDone = false;
  final urlController = TextEditingController();
  final _channel = WebSocketChannel.connect(
    Uri.parse(Constants.hostWs),
  );
  final Set<Factory> gestureRecognizers = [
    Factory(() => EagerGestureRecognizer()),
  ].toSet();

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
  }
  @override
  void dispose() {
    // TODO: implement dispose
    super.dispose();
    _channel.sink.close();
  }
  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    print('inner card'+ widget.urlInnerCard);
    return SafeArea(child:  Container(
        child: Column(
          children: [
            progress < 1.0
                ? LinearProgressIndicator(value: progress)
                : Container(
              padding: const EdgeInsets.all(10.0),
              decoration: BoxDecoration(
                color: Colors.white
              ),
            ),
            Expanded(child:  StreamBuilder(
              stream: _channel.stream,
              builder: (context, snapshot) {
                if(snapshot.hasData) {
                  Map<String, dynamic> payment = jsonDecode(snapshot.data.toString());
                  if(payment['wowpay_order_id'] == widget.orderId){
                    dynamic payload;
                    if(payment['isFailure'] == true || payment['isFailure'] != null){
                      Map<String,dynamic> dataFailure = {
                        "code":payment['wowpay_transaction_info']["paymentrs_info"]["txn_statuscode"],
                        "message":payment['wowpay_transaction_info']["paymentrs_info"]["provider_desc"],
                      };
                      payload = PaymentFailureResponse.fromMap(dataFailure);
                        WowpayState.emitter.emit(Wowpay.EVENT_PAYMENT_ERROR,null,payload);
                      if (Platform.isAndroid) {
                        // Android-specific code
                        Future.delayed(Duration.zero, () async {
                          setState(() {
                            isDone=true;
                          });
                        });
                      } else if (Platform.isIOS) {
                        Navigator.pop(context);
                        // iOS-specific code
                      }

                    }else if(payment['isSuccess'] == true || payment['isSuccess'] != true){
                      Map<String,dynamic> dataSuccess = {
                        "wowpay_order_id":payment['wowpay_order_id'],
                        "wowpay_channel_code":payment['wowpay_transaction_info']["channel_code"],
                        "wowpay_signature":payment['wowpay_signature'],
                      };
                      payload = PaymentSuccessResponse.fromMap(dataSuccess);
                      WowpayState.emitter.emit(Wowpay.EVENT_PAYMENT_SUCCESS,null,payload);
                      if (Platform.isAndroid) {
                        // Android-specific code
                        Future.delayed(Duration.zero, () async {
                          setState(() {
                            isDone=true;
                          });
                        });
                      } else if (Platform.isIOS) {
                        Navigator.pop(context);
                        // iOS-specific code
                      }
                    }

                  }
                }
                return InAppWebView(
                  gestureRecognizers: <Factory<OneSequenceGestureRecognizer>>{
                    Factory<VerticalDragGestureRecognizer>(
                          () => VerticalDragGestureRecognizer(),
                    ),
                  } ,
                  key: webViewKey,
                  initialUrlRequest:
                  URLRequest(url: Uri.parse(widget.urlInnerCard)),
                  initialOptions: options,
                  onWebViewCreated: (controller) async {
                    webViewController = controller;
                  },
                  onProgressChanged:(controller, progress)async {
                    print('progress $progress');
                    if(isDone == true){
                      Navigator.pop(context);
                    }else{
                      setState(() {
                        this.progress = progress / 100;
                        urlController.text = this.url;
                      });
                    }


                  },
                );
              },
            ),
            ),

          ],
        )

    ));
  }
}