import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:wowpay_flutter/blocs/payment_bloc.dart';
import 'package:wowpay_flutter/events/payment_event.dart';
import 'package:wowpay_flutter/states/payment_state.dart';
import 'package:flutter_user_agent/flutter_user_agent.dart';
import 'package:wowpay_flutter/utils/constants.dart';
import 'package:modal_bottom_sheet/modal_bottom_sheet.dart';
import 'package:wowpay_flutter/views/payment/modal_payment.dart';
import 'package:wowpay_flutter/wowpay_flutter.dart';
class PaymentWebView extends StatefulWidget {

  final Widget child;
  final dynamic paymentState;
  PaymentWebView({required this.child,required this.paymentState});
  @override
  _PaymentWebView createState() => new _PaymentWebView();
}
class _PaymentWebView extends State<PaymentWebView> {
  late String encValue;
  late String typeCard;
  late String orderId;
  String _webUserAgent = '<unknown>';

  Map<String,dynamic> handleBrowser(){
    Locale myLocale = Localizations.localeOf(context);
    DateTime local_datetime = DateTime.now();
    var timezoneName = local_datetime.timeZoneName;
    var time_diff = local_datetime.timeZoneOffset.inMinutes;
    String language = "${myLocale.languageCode.toString().toLowerCase()}-${myLocale.countryCode.toString().toLowerCase()}";
    double width = MediaQuery.of(context).size.width;
    double height = MediaQuery.of(context).size.height;
      return {
        "UserAgent": _webUserAgent,
        "Language":language,
        "ColorDepth": 32,
        "ScreenWidth": width.toInt(),
        "ScreenHeight": height.toInt(),
        "TimeZoneOffSet":timezoneName.contains('+')? -time_diff: time_diff,
        "JavaEnabled": false
      };
  }
  dynamic handleBody(String body) {
    Map <String,dynamic> options = jsonDecode(body);
    options.remove("typeCard");
    options["browser_info"] =  handleBrowser();
  print(handleBrowser().toString());
    return jsonEncode(options);
  }
  Future<void> initUserAgentState() async {
    String userAgent, webViewUserAgent;
    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      userAgent = await FlutterUserAgent.getPropertyAsync('userAgent');
      await FlutterUserAgent.init();
      webViewUserAgent = FlutterUserAgent.webViewUserAgent;
    } on PlatformException {
      userAgent = webViewUserAgent = '<error>';
    }

    if (!mounted) return;

    setState(() {
      _webUserAgent = webViewUserAgent;
    });
  }
  String getTypeCard(String body) {
    Map <String,dynamic> options = jsonDecode(body);
    return options["typeCard"];
  }
  String getOrderId(String body) {
    Map <String,dynamic> options = jsonDecode(body);
    return options["order_id"];
  }
  @override
  void initState() {
    super.initState();
    initUserAgentState();
    // getColorDepth();
    WowpayState.emitter.on("onOpen", null, (ev, context2) {
      setState(() {
        typeCard=getTypeCard(ev.eventData.toString());
        orderId=getOrderId(ev.eventData.toString());

      });
      BlocProvider.of<PaymentBloc>(context)
          .add(PaymentEventRequested(options:handleBody(ev.eventData.toString())));
    });
    WowpayState.emitter.on("onFinish", null, (ev, context2) {
      print("on Finish roi");
      Future.delayed(Duration.zero,() {
        handleModal();
      });
    });
  }

@override
  void didUpdateWidget(covariant PaymentWebView oldWidget) {
    // TODO: implement didUpdateWidget
    super.didUpdateWidget(oldWidget);
    if(oldWidget.paymentState != widget.paymentState || (oldWidget.paymentState is PaymentStateSuccess && widget.paymentState is PaymentStateSuccess)){
      final paymentState = widget.paymentState;
      if(paymentState is PaymentStateSuccess){
        setState(() {
          encValue =paymentState.payment.toString();
        });
        WowpayState.emitter.emit("onFinish",null,"ok");
      }


    }
  }
  @override
  void dispose() {
    super.dispose();

  }
  void handleModal(){
    showCupertinoModalBottomSheet(
      expand: true,
      context: context,
      backgroundColor: Colors.transparent,
      builder: (context) => ModalPayment(urlInnerCard:Constants.innerCardUrl(typeCard, encValue),orderId: orderId),
    );
  }
  @override
  Widget build(BuildContext context) {
     return widget.child;

  }
}


