
import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:wowpay_flutter/wowpay_widget.dart';
import 'package:eventify/eventify.dart';

class PaymentSuccessResponse {
  String? channelCode;
  String? orderId;
  String? signature;

  PaymentSuccessResponse(this.channelCode, this.orderId, this.signature);

  static PaymentSuccessResponse fromMap(Map<dynamic, dynamic> map) {
    String? channelCode = map["wowpay_channel_code"];
    String? signature = map["wowpay_signature"];
    String? orderId = map["wowpay_order_id"];
    return new PaymentSuccessResponse(channelCode, orderId, signature);
  }
}

class PaymentFailureResponse {
  int? code;
  String? message;

  PaymentFailureResponse(this.code, this.message);

  static PaymentFailureResponse fromMap(Map<dynamic, dynamic> map) {
    var code = map["code"] as int?;
    var message = map["message"] as String?;
    return new PaymentFailureResponse(code, message);
  }
}
class Wowpay {
  static const EVENT_PAYMENT_SUCCESS = 'payment.success';
  static const EVENT_PAYMENT_ERROR = 'payment.error';
  Map<String,dynamic> options;
  void _validation (options){
    if(!options.containsKey("api_key")){
      throw("api_key field is required for params options wowpay_flutter");
    }else if(!options.containsKey("amount")){
      throw("amount field is required for params options wowpay_flutter");
    }else if(!options.containsKey("currency")){
      throw("currency field is required for params options wowpay_flutter");
    }else if(!options.containsKey("order_id")){
      throw("order_id field is required for params options wowpay_flutter");
    }else if(!options.containsKey("customer")){
      throw("customer field is required for params options wowpay_flutter");
    }else if(!options.containsKey("order_info")){
      throw("order_info field is required for params options wowpay_flutter");
    }else if(!options.containsKey("user_id")){
      throw("user_id field is required for params options wowpay_flutter");
    }
    if(!options["customer"].containsKey("title")){
      throw("customer.title field is required for params options wowpay_flutter");
    }else if(!options["customer"].containsKey("first_name")){
      throw("customer.first_name field is required for params options wowpay_flutter");
    }else if(!options["customer"].containsKey("last_name")){
      throw("customer.last_name field is required for params options wowpay_flutter");
    }else if(!options["customer"].containsKey("mobile")){
      throw("customer.mobile field is required for params options wowpay_flutter");
    }else if(!options["customer"].containsKey("email")){
      throw("customer.email field is required for params options wowpay_flutter");
    }else if(!options["customer"].containsKey("customer_type")){
      throw("customer.customer_type field is required for params options wowpay_flutter");
    }else if(!options["customer"].containsKey("islead_pax")){
      throw("customer.islead_pax field is required for params options wowpay_flutter");
    }else if(!options["customer"].containsKey("address")){
      throw("customer.address field is required for params options wowpay_flutter");
    }else if(!options["customer"].containsKey("city")){
      throw("customer.city field is required for params options wowpay_flutter");
    }else if(!options["customer"].containsKey("postcode")){
      throw("customer.postcode field is required for params options wowpay_flutter");
    }else if(!options["customer"].containsKey("country")){
      throw("customer.country field is required for params options wowpay_flutter");
    }
    if(!options["order_info"].containsKey("item_list")){
      throw("order_info.item_list field is required for params options wowpay_flutter");
    }else if(!options["order_info"].containsKey("order_desc")){
      throw("order_info.order_desc field is required for params options wowpay_flutter");
    }

  }
  Wowpay(this.options){
    _validation(this.options);


    options = this.options;
  }

  void onOpen(String typeCard) {
    options["typeCard"] = typeCard;
    String jsonOptions = jsonEncode(options);
    WowpayState.emitter.emit("onOpen", null, jsonOptions);
  }

  void on(String event, Function handler) {
    EventCallback cb = (event, cont) {
      handler(event.eventData);
    };
    WowpayState.emitter.on(event, null, cb);
  }

  /// Clears all event listeners
  void clear() {
    WowpayState.emitter.clear();
  }

}
class WowpayInit extends StatefulWidget{
  late Widget child;
  WowpayInit({required this.child});
  @override
  State<StatefulWidget> createState() {
    // TODO: implement createState
    return WowpayState(child: child);
  }
}
class WowpayState extends State<WowpayInit>{

  static EventEmitter emitter = new EventEmitter();

  late Widget child;
  WowpayState({required this.child});
  @override
  void initState() {
    // TODO: implement initState
    super.initState();

  }
  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return Stack(children: [
      WowpayWidget(child: child)
    ],);
  }
}